/*
 * msh/msh.c --
 *
 *      This file contains the simple and stupid shell (msh).
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include <assert.h>

static const char *progname = "msh";

static void
show_prompt(void)
{
    printf("%s > ", progname);
    fflush(stdout);
}

static char*
next_token(char **s)
{
    char *token, *p;

    assert(s && *s);
    
    for (p = *s; *p && isspace(*p); p++) {
        *p = '\0';
    }
    token = p;
    for (; *p && !isspace(*p); p++) ;
    for (; *p && isspace(*p); p++) {
        *p = '\0';
    }
    *s = p;
    return token;
}

static void
read_command(FILE *stream, int *argc, char ***args)
{
#ifndef BUFLEN
#define BUFLEN 512
#endif
    static char line[BUFLEN];
    static char* argv[BUFLEN];
    char *p;

    assert(argc && args);

    memset((char *) argv, 0, sizeof(argv));
    p = fgets(line, sizeof(line), stream);
    for (*argc = 0; p && *p; (*argc)++) {
        argv[*argc] = next_token(&p);
    }
    *args = argv;
}

int
main()
{
    pid_t pid;
    int status;
    int argc;
    char **argv;

    while (1) {
        show_prompt();
        read_command(stdin, &argc, &argv);
        if (argv[0] == NULL || strcmp(argv[0], "exit") == 0) {
            break;
        }
        if (strlen(argv[0]) == 0) {
            continue;
        }
        pid = fork();
        if (pid == -1) {
            fprintf(stderr, "%s: fork: %s\n", progname, strerror(errno));
            continue;
        }
        if (pid == 0) {			/* child */
            execvp(argv[0], argv);
            fprintf(stderr, "%s: execvp: %s\n", progname, strerror(errno));
            _exit(EXIT_FAILURE);
        } else {			/* parent */
            if (waitpid(pid, &status, 0) == -1) {
		fprintf(stderr, "%s: waitpid: %s\n", progname, strerror(errno));
	    }
        }
    }

    return EXIT_SUCCESS;
}
